@@ -6,12 +6,10 @@
 #define _C910_PLATFORM_H_
 
 #define C910_HART_COUNT   16
-#define C910_HART_STACK_SIZE   8192
 
 #define SBI_THEAD_FEATURES	\
-	(SBI_PLATFORM_HAS_SCOUNTEREN | \
-	 SBI_PLATFORM_HAS_MCOUNTEREN | \
-	 SBI_PLATFORM_HAS_MFAULTS_DELEGATION)
+	 (SBI_PLATFORM_HAS_MFAULTS_DELEGATION | \
+	 SBI_PLATFORM_HAS_HART_SECONDARY_BOOT)
 
 #define CSR_MCOR         0x7c2
 #define CSR_MHCR         0x7c1
@@ -22,8 +20,6 @@
 #define CSR_MRMR         0x7c6
 #define CSR_MRVBR        0x7c7
 
-#define SBI_EXT_VENDOR_C910_BOOT_OTHER_CORE    0x09000003
-
 #define C910_PLIC_CLINT_OFFSET     0x04000000  /* 64M */
 #define C910_PLIC_DELEG_OFFSET     0x001ffffc
 #define C910_PLIC_DELEG_ENABLE     0x1
